//
//  CameraRollViewController.m
//  CameraRoll
//
//  Created by Louis Romero on 10/02/11.
//  Copyright 2011 Joyous Code. All rights reserved.
//

#import "CameraRollViewController.h"
#import "Constants.h"


#pragma mark Private interface

@interface CameraRollViewController (PrivateMethods)

- (CGRect)frameForRowAtIndex:(NSUInteger)index positionInRow:(NSUInteger)positionInRow;
- (void)loadImagesInBackground;

@end


#pragma mark -
#pragma mark Implementation

@implementation CameraRollViewController

@synthesize scrollView, imageViews;


#pragma mark -
#pragma mark Private methods

- (CGRect)frameForRowAtIndex:(NSUInteger)index positionInRow:(NSUInteger)positionInRow {
    CGFloat width = [[self view] bounds].size.width / IMAGES_PER_LINE;
    CGFloat height = width; // We display square image views.
    
    CGFloat x = width  * positionInRow;
    CGFloat y = height * index;
    
    return CGRectMake(x, y, width, height);
}

- (void)loadImagesInBackground {
    dispatch_queue_t queue = dispatch_queue_create("fr.joyouscode.cameraroll.imageloading", NULL);

    for (NSString *name in [self imageViews]) { // Since we're using fast enumeration this way here,
                                                // the images are displayed in 'random' order. It's 
                                                // actually funny in this tiny demo app, so we'll 
                                                // leave it this way.
        dispatch_async(queue, ^{
            [NSThread sleepForTimeInterval:IMAGE_LOADING_DELAY];
            
            UIImage *image = [UIImage imageNamed:name];

            dispatch_async(dispatch_get_main_queue(), ^{
                [[[self imageViews] objectForKey:name] setImage:image];
            });
        });
    }
    
    dispatch_release(queue);
}


#pragma mark -
#pragma mark View lifecycle

- (void)viewDidLoad {
    [super viewDidLoad];
    
    // Custom settings for the scrollView.
    CGFloat width = [[self view] bounds].size.width / IMAGES_PER_LINE;
    CGFloat height = width; // We display square image views.
    [[self scrollView] setContentSize:CGSizeMake(width, NUMBER_LINES * height)];
    
    // We set the image views and keep them in the dictionary.
    NSMutableDictionary *dict = [[NSMutableDictionary alloc] init];
    UIImageView *imageView;
    for (NSUInteger i = 0; i < NUMBER_LINES; ++i) {
        for (NSUInteger j = 0; j < IMAGES_PER_LINE; ++j) {
            // We create and add the UIImageView to the UIScrollView.
            imageView = [[UIImageView alloc] initWithFrame:[self frameForRowAtIndex:i positionInRow:j]];
            [imageView setContentMode:UIViewContentModeScaleAspectFill];
            [imageView setClipsToBounds:YES];
            [[self scrollView] addSubview:imageView];
            
            // We keep a reference to the image view. The key is the name of the file.
            [dict setObject:imageView forKey:[NSString stringWithFormat:@"image%d.jpeg", i*IMAGES_PER_LINE + j]];
            
            [imageView release], imageView = nil;
        }
    }
    [self setImageViews:dict];
    [dict release], dict = nil;
    
    // We load the images in the background.
    [self loadImagesInBackground];
}

- (void)viewDidUnload {
    [super viewDidUnload];
    
    [self setScrollView:nil];
    [self setImageViews:nil];
}


#pragma mark -
#pragma mark Memory management

- (void)dealloc {
    [scrollView release];
    [imageViews release];
    
    [super dealloc];
}

@end
